#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from PyQt4 import QtCore,QtGui
from PyQt4.QtGui import QLabel
from Driza.iu.ui.doperaciones import Ui_DialogoOperaciones
from Driza.gestores import GestorMensajesCategorias
import logging
log = logging.getLogger("Driza.iu.dialogs.doperaciones")

class DOperaciones(QtGui.QDialog):
    """Dialogo  de operaciones. Muestra las operaciones disponibles
    Y ademas muestra el woperaciones de la operación actualmente seleccionada
    """

    def __init__(self,parent,gestoroperaciones,vsalida):
        self.gestorcategorias=GestorMensajesCategorias()
        QtGui.QDialog.__init__(self,parent)
        self.ui=Ui_DialogoOperaciones()
        self.ui.setupUi(self)
        self.__gestoroperaciones=gestoroperaciones
        self.__initwidgets()
        self.__initarbol()
        self.__conexiones()
        self.__vsalida=vsalida
        self.__noimplementado=QLabel(None)
        self.__noimplementado.setText("No implementado")
        self.__condicionesincorrectas=QLabel(None)
        self.__condicionesincorrectas.setText("No se dan las condiciones adecuadas")
        self.ui.stackedWidget.addWidget(self.__condicionesincorrectas)
        self.ui.treeWidget.setRootIsDecorated(True)
        #self.ui.treeWidget.addColumn("Operaciones")
        self.__operacion=None #Guarda la operacion actual
    
    def mostrar(self,operacion):
        self.__operacion=operacion
        self.show()

    def showEvent(self,ev):
        self.__myUpdate()
        DialogoOperaciones.showEvent(self,ev)

    def accept(self):
        """Acepta el dialogo. Coge la operacion con la que esta trabajando el usuario e inicia el procedimiento de calculo"""
        from Driza.excepciones import OpcionesIncorrectaException
        import rpy
        nombre=unicode(self.ui.treeWidget.currentItem().text(0).latin1(),'iso-8859-1')
        try:
            resultado=self.__gestoroperaciones[nombre].procedimiento()
        except OpcionesIncorrectaException:
            QErrorMessage(self,"error").message(u"Las opciones no son correctas")
        except IndexError:
            QErrorMessage(self,"error").message(u"Seleccion Incorrecta")
            log.exception("excepcion capturada")
        except KeyError:
            QErrorMessage(self,"error").message(u"Hay elementos de la salida sin definir(o mal definidos) en la operación")
            log.exception("Excepción Generada por un módulo de operaciones")
        except rpy.RException:
            QErrorMessage(self,"error").message(u"R devolvio un error")
            log.exception("Excepción de RPY")
        except AssertionError:
            QErrorMessage(self,"error").message(u"Error desconocido")
        else:
            self.__vsalida.appRes(resultado) #Añadir a la salida el resultado
            self.__vsalida.hide() #TODO averiguar como hacer para que recupere el foco sin ocultar la ventana
            self.__vsalida.show()

    def __cambiarelemento(self):
        self.__operacion=unicode(self.ui.treeWidget.currentItem().text(0).latin1(),'iso-8859-1')
        self.__showwidget()


    def __initwidgets(self):
        """Añade los widgets en el widgetstack"""
        for operacion in self.__gestoroperaciones.values():
            self.ui.stackedWidget.addWidget(operacion.widget)

    def __initarbol(self):
        """Inicializa el arbol con todas las operaciones conocidas
        """
        from PyQt4.QtGui import QTreeWidgetItem
        from PyQt4.QtCore import QStringList
        #TODO En futuras implementaciones la lista se generará dinámicamente
        #self.ui.treeWidget.clear()
        descriptivo=QTreeWidgetItem(self.ui.treeWidget,QStringList(["Descriptivos"]))
        self.gestorcategorias["Descriptivos"]="En esta categoria se encuentran los mensajes descriptivos"
        self.ui.stackedWidget.addWidget(self.gestorcategorias["Descriptivos"])
        estadisticos=QTreeWidgetItem(descriptivo,[u"Estadísticos"])
        QTreeWidgetItem(estadisticos,["Descriptivo"])
        QTreeWidgetItem(descriptivo,[u"Tabla de frecuencias"])
        graficos=QTreeWidgetItem(descriptivo,[u"Gráficas"])
        QTreeWidgetItem(graficos,["Histograma"])
        QTreeWidgetItem(graficos,[u"Diagrama de dispersión"])
        QTreeWidgetItem(graficos,[u"Diagrama de barras"])

       # cmedias=QTreeWidgetItem(self.ui.treeWidget,u"Comparación de medias")
       # QTreeWidgetItem(cmedias,u"Comparación de medias simple")
       # QTreeWidgetItem(cmedias,u"Comparación de medias entre dos variables")
       # QTreeWidgetItem(cmedias,u"Anova Simple")
       # cprop=QTreeWidgetItem(self.ui.treeWidget,u"Comparación de proporciones")
       # QTreeWidgetItem(cprop,u"Comparación de proporciones simple")
       # QTreeWidgetItem(cprop,u"Media de dos muestras independientes")
       # cvar=QTreeWidgetItem(self.ui.treeWidget,u"Comparación de Varianzas")
       # QTreeWidgetItem(cvar,u"Comparación de Varianzas entre dos variables")
       # QTreeWidgetItem(cvar,u"Comparación de Varianzas entre múltiples variables")
       # bajuste=QTreeWidgetItem(self.ui.treeWidget,"Bondad de ajuste")
       # QTreeWidgetItem(bajuste,"KS")
       # QTreeWidgetItem(bajuste,"Ji Cuadrado")
       # rlineal=QTreeWidgetItem(self.ui.treeWidget,u"Regresión Lineal")
       # QTreeWidgetItem(rlineal,u"Regresión")
       # distribuciones=QTreeWidgetItem(self.ui.treeWidget,"Distribuciones")
       # QTreeWidgetItem(distribuciones,u"Distribución Normal")


    def __conexiones(self):
        from PyQt4.QtCore import SIGNAL
        self.connect(self.ui.treeWidget,SIGNAL("selectionChanged ()"),self.__cambiarelemento)
        self.connect(self.ui.pushButton2,SIGNAL("clicked()"),self.reject)
        self.connect(self.ui.pushButton1,SIGNAL("clicked()"),self.accept)

    def __myUpdate(self):
        self.__showarbol()
        self.__showwidget()

    def __showarbol(self):
        """Muestra el arbol segun el contenido de gestoroperaciones"""
        #FIXME No deberia ser rellenado más de una vez
        self.ui.treeWidget.ensureItemVisible(self.ui.treeWidget.findItem(self.__operacion,0))
        self.ui.treeWidget.setCurrentItem(self.ui.treeWidget.findItem(self.__operacion,0))

    def __showwidget(self):
        """Muestra el widget asociado a la seleccion actual"""
        nombre=unicode(self.ui.treeWidget.currentItem().text(0).latin1(),'iso-8859-1')
        if self.__gestoroperaciones.has_key(nombre):
            if self.__gestoroperaciones.funcionchequeocondiciones(nombre):
                self.ui.stackedWidget.raiseWidget(self.__gestoroperaciones[nombre].widget)
            else:
                self.ui.stackedWidget.raiseWidget(self.__condicionesincorrectas)
        elif self.gestorcategorias.has_key(nombre):
            self.ui.stackedWidget.raiseWidget(self.gestorcategorias[nombre])
        else:
            self.ui.stackedWidget.raiseWidget(self.__noimplementado)
