#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2006-2007  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from PyQt4 import QtCore,QtGui
from PyQt4.QtGui import QMessageBox,QIcon
from Driza.iu.ui.vprincipal import Ui_VentanaPrincipal
from Driza.iu.dconfig import DConfig
from Driza.iu.dcrevar import DCrevar
from Driza.iu.dbuscar import DBuscar
from Driza.iu.operaciones.doperaciones import DOperaciones
#from Driza.iu.dimportartexto import DImportarTexto
from Driza.iu.dfiltrado import DFiltrado
from Driza.iu.vsalida import VSalida
from grid import Grid
from Driza import gestores
from Driza import funciones
from Driza import operaciones
from Driza import listas
from Driza.datos import datos
from Driza.datos import interfaces
import string
import logging
log = logging.getLogger("Driza.vprincipal")

class VPrincipal(QtGui.QMainWindow):
    """Ventana principal, hereda de vprincipal, que esta generado por el designer. Esta clase es padre de la mayoria de los dialogos, y es la que permite introducir los datos a el usuario"""

    def __init__(self,parent,config,portero,gestorproyectos,gestortemas): 
        """Inicializador de la ventana principal"""
        QtGui.QMainWindow.__init__(self,None)
        self.ui=Ui_VentanaPrincipal()
        self.ui.setupUi(self)
        self.stackedWidget=self.ui.stackedWidget
        gestorpaquetes = funciones.GestorPaquetes()
        gestorsalida = gestores.GestorSalida()
        vsalida = VSalida(self,gestorsalida,gestortemas)
        idf = interfaces.InterfazDatosFicheros(portero)
        idr = interfaces.InterfazDatosR(portero)
        #VARIABLES PUBLICAS
        self.__idu = interfaces.InterfazDatosUsuario(portero,gestorpaquetes)
        self.parent=parent
        self.__config=config
        self.__gestortemas=gestortemas
        gestoroperaciones = operaciones.GestorOperaciones(idr,config)
        #Bloque de inicialización de dialogos
        self.__doperaciones = DOperaciones(self,gestoroperaciones,vsalida)
        self.__dcrevar = DCrevar(self,self.__idu,config,gestorpaquetes)
        self.__dbuscar = DBuscar(self,"busqueda")
        self.__dconfig = DConfig(config,self)
        self.__dfiltro = DFiltrado(self,self.__idu)
        #self.__dimportartexto=DImportarTexto(self,idf)
        self.__portero=portero
        self.__gestorproyectos=gestorproyectos
        self.grid= Grid(self,self.__idu,self.__portero)
        self.stackedWidget.addWidget(self.grid)
        self.stackedWidget.setCurrentWidget(self.grid)
        # Funciones de inicializacion de componentes
        self.__inittoolbar()
        self.__conexiones()
        self.showundoredo()

        #self.setIcon(self.__gestortemas.iconoprograma())

    #FUNCIONES PUBLICAS
    
    def abrirfichero(self,parent=None,filename=None):
        """ Lanza el diálogo de apertura de fichero """
        from Driza.excepciones import FicheroNoExisteException,FicheroErroneoException,FicheroTipoDesconocidoException
        if not parent: parent = self
        if not self.__preguntarproyectomodificado(): return
        if not filename:
            filter=""
            for fmt in listas.extensiones_fichero():
                filter=filter+"%s files (*.%s);;" % (fmt,string.lower(fmt))
            from PyQt4.QtGui import QFileDialog
            from PyQt4.QtCore import QString
            fn = QFileDialog.getOpenFileName(self,"Dialogo abrir fichero","",filter)
            filename = str(fn)
        if filename:
            try:
                self.__gestorproyectos.cargar(filename)
            except FicheroErroneoException:
                QMessageBox.warning(parent,u'Atención', 'El fichero no ha podido ser leido')
            except FicheroTipoDesconocidoException:
                QMessageBox.warning(parent,u'Atención', 'El fichero no ha podido ser leido')
            except AttributeError:
                QErrorMessage(parent,"Error").message(u"Parece que ha intentado cargar un fichero de una versión anterior. Lo siento")
                log.exception("excepcion capturada")
            except:
                QErrorMessage(parent,"Error").message(u"Error desconocido")
            else:
                self.myUpdate()
                self.grid.myUpdate()
                return True
        return False

    def myUpdate(self):
        """Acciones de actualizacion"""
        if self.__gestorproyectos.fichero:
            nfichero= self.__gestorproyectos.fichero
        else:
            nfichero= "Nuevo fichero"
        #TODO Pendiente Qt4
        #self.setCaption("Driza "+self.parent.version()+" - "+nfichero)

    def showEvent(self,ev):
        #self.myUpdate()
        QtGui.QMainWindow.showEvent(self,ev)

    def closeEvent( self, ev ):
        """Acciones que se realizaran cuando el usuario cierre la ventana. Guarda el estado de la configuración, incluyendo los ficheros recientemente abiertos"""
        from Driza.config import InterfazConfig
        if not self.__preguntarproyectomodificado(): return
        self.__config.save()
        QtGui.QWidget.closeEvent( self, ev )

    
    def showundoredo(self):
        """Determina que botones estan activados y cuales no del submenu de edición"""
        if self.__portero.puedoundo():
            self.ui.actionDeshacer.setEnabled(True)
        else:
            self.ui.actionDeshacer.setEnabled(False)

        if self.__portero.puedoredo():
            self.ui.actionRehacer.setEnabled(True)
        else:
            self.ui.actionRehacer.setEnabled(False)

    #FUNCIONES PRIVADAS

    def __importar(self):
        """Importa los datos de un fichero"""
        if not self.__preguntarproyectomodificado(): return
        self.__dimportartexto.show()

    def __preguntarproyectomodificado(self):
        """Pregunta en caso de que haya sido modificado el proyecto si desea ser guardado"""
        if not self.__idu.original():
            rc = QMessageBox.information(self,'Atencion:', 'El proyecto actual ha sido modificado, desea guardarlo?', 'Guardarlo','No guardarlo','Volver',0,1)
            if rc==0:
                self.__guardar()
            elif rc==2:
                return False
        return True


    def __dacerca_de(self):
        """Mensaje acerca de"""
        QMessageBox.about(self,"Acerca de Driza", u"Driza %s es una interfaz estadística\n (C) Néstor Arocha Rodríguez - Tutorizado por Inmaculada luengo Merino\n Distribuido bajo licencia GPL\n----------------------------------\n Betatesters:\n Carlos Mestre Gonzalez\n Luis de Bethencourt Guimerá\n----------------------------------\n Mantenedor del paquete .deb:\n Luis de Bethencourt Guimerá\n" % (self.parent.version()))
    
    def __undo(self):
        """Deshace el último cambio"""
        self.__portero.undo()
        self.grid.myUpdate()
        self.showundoredo()
    
    def __redo(self):
        """Rehace el último cambio"""
        self.__portero.redo()
        self.grid.myUpdate()
        self.showundoredo()

    def __inittoolbar(self):
        """Inicializa la toolbar"""
        #self.setIconSize(QtCore.QSize(64,64))
        #self.ui.toolBar.setIconSize(QtCore.QSize(64,64))
        self.boton1=QtGui.QToolButton(self.ui.toolBar)
        self.ui.toolBar.addWidget(self.boton1)
        self.boton1.setText("ABRIR")
        icono=QIcon()
        icono.addPixmap(self.__gestortemas.iconoabrir())
        self.boton1.setIcon(icono)
        self.boton2=QtGui.QToolButton(self.ui.toolBar)
        self.boton2.setText("GUARDAR")
        icono=QIcon()
        icono.addPixmap(self.__gestortemas.iconoguardar())
        self.boton2.setIcon(icono)
        self.ui.toolBar.addWidget(self.boton2)

    def __conexiones(self):
        """Funcion llamada en el constructor que almacena todas las conexiones accion-funcion"""
        from PyQt4.QtCore import SIGNAL
        self.connect(self.ui.actionSalir,SIGNAL("activated()"),self.__salirprograma)
        self.connect(self.ui.actionNuevo_proyecto,SIGNAL("activated()"),self.__nuevo)
        self.connect(self.ui.actionAbrir_Proyecto,SIGNAL("activated()"),self.abrirfichero)
        self.connect(self.ui.actionGuardar_como,SIGNAL("activated()"),self.__guardarcomo)
        self.connect(self.ui.actionGuardar,SIGNAL("activated()"),self.__guardar)
        self.connect(self.ui.actionImportar,SIGNAL("activated()"),self.__importar)
        self.connect(self.ui.actionConfiguraci_n,SIGNAL("activated()"),self.__dconfig.show)
        self.connect(self.ui.actionFiltrar,SIGNAL("activated()"),self.__dfiltro.show)
        self.connect(self.ui.actionCrear_nuevas_variables,SIGNAL("activated()"),self.__dcrevar.show)
        self.connect(self.ui.actionBuscar,SIGNAL("activated()"),self.__dbuscar.show)
        self.connect(self.ui.actionCopiar,SIGNAL("activated()"),self.__copy)
        self.connect(self.ui.actionCotar,SIGNAL("activated()"),self.__cut)
        self.connect(self.ui.actionPegar,SIGNAL("activated()"),self.__paste)
        self.connect(self.ui.actionDeshacer,SIGNAL("activated()"),self.__undo)
        self.connect(self.ui.actionRehacer,SIGNAL("activated()"),self.__redo)
        self.connect(self.ui.actionBorrar,SIGNAL("activated()"),self.__borrar)
        self.connect(self.ui.actionAcerca,SIGNAL("activated()"),self.__dacerca_de)
        self.connect(self.boton1,SIGNAL("clicked()"),self.abrirfichero)
        self.connect(self.boton2,SIGNAL("clicked()"),self.__guardar)
        #self.__conexionesanalizar()
        #self.connect(self.distribucionesDistribucion_NormalAction,SIGNAL("activated()"),self.__ddistribucionnormal)


    def __conexionesanalizar(self):
        """Las conexiones del submenu "analizar" """
        self.connect(self.analizarMediaGrficosHistogramaAction,SIGNAL("activated()"),self.__dgraficoshistograma)
        self.connect(self.analizarMediaGrficosDiagrama_de_dispersinAction,SIGNAL("activated()"),self.__dgraficosdispersion)
        self.connect(self.analizarMediaDescriptivosAction,SIGNAL("activated()"),self.__dcalculodescriptivo)
        self.connect(self.analizarBondad_de_ajusteJi_cuadradoAction,SIGNAL("activated()"),self.__dcalculojicuadrado)
        self.connect(self.AnalizarKs,SIGNAL("activated()"),self.__dcalculoks)
        self.connect(self.analizarRegresin_linealRegresin_lineal_simpleAction,SIGNAL("activated()"),self.__dcalculoregresion)
        self.connect(self._2Comparacin_con_una_cteAction,SIGNAL("activated()"),self.__dcalculocompmedsim)
        self.connect(self._2Comparacin_entre_dos_variablesAction,SIGNAL("activated()"),self.__dcalculocompmeddob)
        self.connect(self.analizarPruebas_de_hiptesisProporcionesComparacin_con_una_cteAction,SIGNAL("activated()"),self.__dcalculocomppropsim)
        self.connect(self.analizarPruebas_de_hiptesisVarianzaComparacin_entre_dos_variablesAction,SIGNAL("activated()"),self.__dcalculocompvardob)
        self.connect(self.analizarPruebas_de_hiptesisVarianzanew_itemAction,SIGNAL("activated()"),self.__dcalculocompvarmul)
        self.connect(self.analizarMediaTabla_de_frecuenciasAction,SIGNAL("activated()"),self.__dtablafrecuencia)
        self.connect(self.analizarMediaGrficosDiagrama_de_barrasAction,SIGNAL("activated()"),self.__ddiagramabarras)
        self.connect(self._2ANOVAAction,SIGNAL("activated()"),self.__danova)

    def __copy(self):
        """Funcion que copia y borra la seleccion"""
        try:
            self.__copiar()
        except:
            QMessageBox.warning(self,u'Atención', 'Las operaciones de copiado, cortado y pegado no han sido implementadas')
        else:
            self.grid.removeselection() 

    
    def __cut(self):
        """Copia, borra la seleccion y su contenido"""
        self.__portero.guardarestado()
        try:
            self.__copiar()
        except:
            QMessageBox.warning(self,u'Atención', 'Las operaciones de copiado, cortado y pegado no han sido implementadas')
        else:
            self.grid.removeselection(borrardatos=True)
            self.grid.myUpdate()


    def __paste(self):
        """Pega lo que haya sido copiado"""
        self.__portero.guardarestado()
        cb = QApplication.clipboard()
        registro=cb.text().latin1()
        lista=eval(registro)
        pos=self.grid.table1.currentRow()
        lista.reverse() #Escribimos los registros al reves para que queden ordenados
        if pos > self.__idu.numReg():
            for x in range(self.__idu.numReg(),pos):
                self.__idu.appReg()
        #Hacemos el pegado efectivo
        for registro in lista:
            self.__idu.insReg(pos,registro)
        self.grid.myUpdate()


    def __salirprograma(self):
        """Funcion de salida del programa. Guarda la configuracion"""
        from Driza.config import InterfazConfig
        if not self.__preguntarproyectomodificado(): return
        self.__config.save()
        QtGui.qApp.exit(0)


    def __nuevo(self):
        """Accion realizada cuando el usuario clickea en Nuevo"""
        if not self.__preguntarproyectomodificado(): return
        self.__idu.borrartodo()
        self.__gestorproyectos.fichero=None
        self.myUpdate()
        self.grid.myUpdate()


    
    def __guardar(self):
        """Funcion de guardado del fichero. Si el fichero no es conocido se llama a guardarcomo"""
        if self.__gestorproyectos.fichero:
            self.__gestorproyectos.guardar(sobreescribir=True)
            self.__idu.setFlagoriginal()
        else:
            self.__guardarcomo()

    def __guardarcomo(self):
        """Abre un diálogo pidiendo el nombre de archivo y guarda en dicho archivo"""
        filter=""
        for fmt in listas.extensiones_fichero():
            filter=filter+"%s files (*.%s);;" % (fmt,string.lower(fmt))
        from PyQt4.QtGui import QFileDialog
        fn = QFileDialog.getSaveFileName(self,"Dialogo guardarFichero","",filter)
        filename = str(fn)
        if filename:
            from Driza.excepciones import FicheroExisteException,FicheroTipoDesconocidoException
            import re
            extension = re.compile('.*\..*')
            if not extension.match(filename):
                filename+=".driza"
            try:
                self.__gestorproyectos.guardar(filename)
            except FicheroExisteException,fichero:
                rc = QMessageBox.information(self,'Atencion:', 'El fichero'+fichero.fichero+' ya existe' , 'Sobreescribir','Otro nombre','Cancelar',0,1)
                if rc==0:
                    self.__gestorproyectos.guardar(filename,True)
                    self.__idu.setFlagoriginal()
                    self.myUpdate()
                elif rc==1:
                    self.__guardarcomo()
            except FicheroTipoDesconocidoException:
                QMessageBox.warning(self,u'Atención', u'La extensión del fichero es incorrecta.\nPruebe con otra extensión')
                self.__gestorproyectos.fichero=None
            else:
                self.__idu.setFlagoriginal()
                self.myUpdate()

                    
    def __copiar(self):
        """Esta funcion pega el contenido del clipboard en la tabla actual. Es llamada por copy"""
        tablaactual=self.grid.currentPageIndex()
        assert(tablaactual == 0) #Cambiar por una excepcion más concreta
        lista=self.grid.listaseleccion()
        cadena=repr(lista)
        cb = QApplication.clipboard()
        cb.setData(QTextDrag(cadena))
            

    def __dcalculodescriptivo(self):
        """Muestra el dialogo de calculo descriptivo"""
        self.__doperaciones.mostrar("Descriptivo")

    def __dtablafrecuencia(self):
        """Muestra"""
        self.__doperaciones.mostrar(u"Tabla de frecuencias")

    def __dcalculojicuadrado(self):
        """Muestra el dialogo ji cuadrado"""
        self.__doperaciones.mostrar("Ji Cuadrado")

    def __ddiagramabarras(self):
        """Muestra"""
        self.__doperaciones.mostrar(u"Diagrama de barras")

    def __dcalculoks(self):
        """Muestra el dialogo ks"""
        self.__doperaciones.mostrar("KS")

    def __dcalculoregresion(self):
        """Muestra el dialogo de regresion"""
        self.__doperaciones.mostrar(u"Regresión")

    def __dgraficoshistograma(self):
        """Muestra el la seccion de creación de histogramas"""
        self.__doperaciones.mostrar("Histograma")

    def __dgraficosdispersion(self):
        """Muestra el la seccion de creación de histogramas"""
        self.__doperaciones.mostrar(u"Diagrama de dispersión")

    def __ddistribucionnormal(self):
        """Muestra la distribucion normal"""
        self.__doperaciones.mostrar(u"Distribución Normal")

    def __dcalculocompmedsim(self):
        self.__doperaciones.mostrar(u"Comparación de medias simple")
        
    def __dcalculocompmeddob(self):
        self.__doperaciones.mostrar(u"Media de dos muestras independientes")

    def __dcalculocomppropsim(self):
        self.__doperaciones.mostrar(u"Comparación de proporciones simple")
        
    def __dcalculocomppropdob(self):
        self.__doperaciones.mostrar(u"Comparación de proporciones entre dos variables")

    def __dcalculocompvardob(self):
        self.__doperaciones.mostrar(u"Comparación de Varianzas entre dos variables")

    def __dcalculocompvarmul(self):
        self.__doperaciones.mostrar(u"Comparación de Varianzas entre múltiples variables")

    def __danova(self):
        self.__doperaciones.mostrar(u"Anova Simple")

    def __borrar(self):
        self.grid.removeselection(borrardatos=True)
        self.grid.myUpdate()
