#!/usr/bin/python
# -*- coding: utf-8 -*-
#

#Copyright (C) 2006-2007  Néstor Arocha Rodríguez, Inmaculada Luengo Merino

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""Funciones para la creación/edición de variables"""

class GestorPaquetes(dict):
    """
    Gestiona los paquetes que contendran las funciones. Estos paquetes son 
    equivalentes al módulo de python que los contiene
    """
    def __init__(self):
        dict.__init__(self)
        basicas = Paquete("basicas")
        self.__setitem__("basicas", basicas)

    def __getitem__(self, clave):
        """Redefine la función del diccionario.
        Si no encuentra el paquete asociado a la clave, busca en las funciones directamente
        """
        try:
            resultado = dict.__getitem__(self, clave)
        except KeyError:
            funcion = self.obtener_funcion(clave)
            if funcion: 
                return funcion
            else: 
                raise KeyError
        else:
            return resultado

    def lista_funciones(self, prefijo = True):
        """Devuelve una lista con todos los paquetes y todas las funciones
        Prefijo determina si en el nombre se incluye la familia de la función
        """
        if prefijo:
            lista = []
            for paquete in self.items():
                for funcion in paquete[1].keys():
                    lista.append(paquete[1].nombrereal + "." + funcion)
        else:
            for paquete in self.items():
                lista = [funcion for funcion in paquete[1].keys()]
        return lista

    def obtener_funcion(self, texto):
        """Busca una función con el nombre indicado en todos los paquetes, 
        y la devuelve. Si no encuentra nada, devuelve False"""
        for paquete in self.items():
            for funcion in paquete[1].keys():
                if funcion == texto:
                    return paquete[1][funcion]
        return False

              
class Paquete(dict):
    """Representa un paquete. 
    Un paquete es un conjunto de funciones. 
    El acceso a los módulos de las funciones se hace como un diccionario
    """
    def __init__(self, nombre):
        """Inicializa el paquete, cargando como diccionario todos sus módulos"""
        dict.__init__(self)
        self.nombrereal = nombre # El nombre real del paquete
        self.__cargar_modulos()

    def __cargar_modulos(self):
        """Carga todas las funciones pertenecientes al paquete como módulo"""
        cargarmodulo = lambda f: __import__(f, None, None, ["Driza.carga.funciones"])
        ruta = "Driza.carga.funciones." + self.nombrereal
        modulo = cargarmodulo(ruta)
        listamiembros = dir(modulo)
        import re
        principal = re.compile('^[A-Z]+')
        nuevalista = filter(principal.match, listamiembros)
        del nuevalista[nuevalista.index("Funcion")] #El único elemento que no interesa
        nuevalista2 = []
        for nombremodulo in nuevalista:
            nombre = "modulo" + "." + nombremodulo + "()"
            nuevalista2.append(eval(nombre))
        for indicepaquete in range(len(nuevalista)):
            self.__setitem__(nuevalista[indicepaquete], \
                    nuevalista2[indicepaquete]) 
            #Ponemos valor al diccionario

class Funcion:
    """Clase padre de todas las funciones del diálogo de creación de variables"""
    def __init__(self, nombre):
        """Todas las clases hijas deben reescribir el valor de estos atributos"""
        self.nombre = nombre

    def nombre_completo(self):
        """Devuelve el nombre completo del módulo, con su paquete"""
        return self.__module__.split(".")[-1] + "." + self.nombre

